//+------------------------------------------------------------------+
//|                                                          KDJ.mq4 |
//|                                     Copyright? 2009, Walter Choy |
//|                                   this version made at forex-tsd |
//+------------------------------------------------------------------+
#property copyright "Copyright? 2009, Walter Choy"
#property link      ""

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 Red
#property indicator_color2 Blue
#property indicator_color3 Green
#property indicator_level1 0
#property indicator_level2 20
#property indicator_level3 80
#property indicator_level4 100

//---- input parameters
extern ENUM_TIMEFRAMES TimeFrame        = PERIOD_CURRENT;
extern int             nPeriod          = 9;
extern double          factor_1         = 0.6666666;
extern double          factor_2         = 0.3333333;
extern bool            ShowArrows       = true;
extern string          arrowsIdentifier = "KDJ Arrows1";
extern double          arrowsUpperGap   = 1.0;
extern double          arrowsLowerGap   = 1.0;
extern color           arrowsUpColor    = LimeGreen;
extern color           arrowsDnColor    = Red;
extern color           arrowsUpCode     = 241;
extern color           arrowsDnCode     = 242;
extern int             arrowsUpSize     = 2;
extern int             arrowsDnSize     = 2;
extern bool            arrowsOnFirst    = true;
extern bool            interpolate      = true;

//---- buffers
double percentK[];
double percentD[];
double percentJ[];
double RSV[];
double trend[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   IndicatorBuffers(5);

//---- indicators
   SetIndexBuffer(0,percentK); //red
   SetIndexLabel(0, "%K");
   SetIndexBuffer(1,percentD);//blue
   SetIndexLabel(1, "%D");
   SetIndexBuffer(2,percentJ); //green
   SetIndexLabel(2, "%J");
   SetIndexBuffer(3,RSV);
   SetIndexBuffer(4,trend); 
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);
      IndicatorShortName(timeFrameToString(TimeFrame)+"  KDJ");
   
return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   deleteArrows();
    
return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1); 
         if (returnBars) { percentK[0] = limit+1; return(0); }
         if (TimeFrame != Period())
         {
            limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
            for (int i=limit; i>=0; i--)
            {
               int y = iBarShift(NULL,TimeFrame,Time[i]);               
                  percentK[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,nPeriod,factor_1,factor_2,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,arrowsOnFirst,0,y);
                  percentD[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,nPeriod,factor_1,factor_2,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,arrowsOnFirst,1,y);
                  percentJ[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,nPeriod,factor_1,factor_2,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,arrowsOnFirst,2,y);
                  
                  //
                  //
                  //
                  //
                  //
      
                  if (!interpolate || y==iBarShift(NULL,TimeFrame,Time[i-1])) continue;

                  //
                  //
                  //
                  //
                  //

                  datetime time = iTime(NULL,TimeFrame,y);
                     for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
                     for(int j = 1; j < n; j++)
                     {
                        percentK[i+j] = percentK[i] + (percentK[i+n] - percentK[i]) * j/n;
                        percentD[i+j] = percentD[i] + (percentD[i+n] - percentD[i]) * j/n;
                        percentJ[i+j] = percentJ[i] + (percentJ[i+n] - percentJ[i]) * j/n;
                     }                        
            }
         return(0);                
         }

   //
   //
   //
   //
   //
     
   for( i=limit; i>=0; i--)
     {
         double Cn = iClose(NULL,0,i); double Ln = iClose(NULL,0,i); double Hn = iClose(NULL,0,i);
         for(int k=0; k<nPeriod; k++){
            if (Ln > iLow(NULL,0,i+k))  Ln = iLow(NULL,0,i+k);
            if (Hn < iHigh(NULL,0,i+k)) Hn = iHigh(NULL,0,i+k);
         }

         if (Hn-Ln != 0) RSV[i] = (Cn-Ln)/(Hn-Ln)*100; else RSV[i] = 50;

         if (i >= (Bars - nPeriod)) {
            percentK[i] = factor_1 * 50 + factor_2 * RSV[i];
            percentD[i] = factor_1 * 50 + factor_2 * percentK[i];
         } else {
            percentK[i] = factor_1 * percentK[i+1] + factor_2 * RSV[i];
            percentD[i] = factor_1 * percentD[i+1] + factor_2 * percentK[i];
         }
         percentJ[i] = 3 * percentD[i] - 2 * percentK[i];
         trend[i] = trend[i+1];
         if (percentK[i]>percentD[i] && percentK[i]>percentJ[i]) trend[i] = 1;
         if (percentK[i]<percentD[i] && percentK[i]<percentJ[i]) trend[i] =-1;
         manageArrow(i); 
     }
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (ShowArrows)
   {
      deleteArrow(Time[i]);
      if (trend[i]!=trend[i+1])
      {
         if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,arrowsUpSize,false);
         if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode,arrowsDnSize, true);
      }
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,int theWidth, bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   int    add  = 0; if (!arrowsOnFirst) add = _Period*60-1;
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i]+add,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_WIDTH,theWidth);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+ arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}